@@ -75,8 +75,10 @@ pub struct CommitParser {
 	pub body:          Option<Regex>,
 	/// Group of the commit.
 	pub group:         Option<String>,
-	/// Scope of the commit.
+	/// Default scope of the commit.
 	pub default_scope: Option<String>,
+	/// Commit scope for overriding the default scope.
+	pub scope:         Option<String>,
 	/// Whether to skip this commit group.
 	pub skip:          Option<bool>,
 }
@@ -86,9 +88,11 @@ pub struct CommitParser {
 pub struct CommitPreprocessor {
 	/// Regex for matching a text to replace.
 	#[serde(with = "serde_regex")]
-	pub pattern: Regex,
+	pub pattern:         Regex,
 	/// Replacement text.
-	pub replace: String,
+	pub replace:         Option<String>,
+	/// Command that will be run for replacing the commit message.
+	pub replace_command: Option<String>,
 }
 
 /// Parser for extracting links in commits.
