@@ -41,8 +41,9 @@ fn generate_changelog() -> Result<()> {
 		conventional_commits:  Some(true),
 		filter_unconventional: Some(true),
 		commit_preprocessors:  Some(vec![CommitPreprocessor {
-			pattern: Regex::new(r#"\(fixes (#[1-9]+)\)"#).unwrap(),
-			replace: String::from("[closes Issue${1}]"),
+			pattern:         Regex::new(r#"\(fixes (#[1-9]+)\)"#).unwrap(),
+			replace:         Some(String::from("[closes Issue${1}]")),
+			replace_command: None,
 		}]),
 		commit_parsers:        Some(vec![
 			CommitParser {
@@ -50,6 +51,7 @@ fn generate_changelog() -> Result<()> {
 				body:          None,
 				group:         Some(String::from("shiny features")),
 				default_scope: None,
+				scope:         None,
 				skip:          None,
 			},
 			CommitParser {
@@ -57,6 +59,15 @@ fn generate_changelog() -> Result<()> {
 				body:          None,
 				group:         Some(String::from("fix bugs")),
 				default_scope: None,
+				scope:         None,
+				skip:          None,
+			},
+			CommitParser {
+				message:       Regex::new("^test").ok(),
+				body:          None,
+				group:         None,
+				default_scope: None,
+				scope:         Some(String::from("tests")),
 				skip:          None,
 			},
 		]),
@@ -110,6 +121,10 @@ fn generate_changelog() -> Result<()> {
 					String::from("hjkl12"),
 					String::from("chore: do boring stuff"),
 				),
+				Commit::new(
+					String::from("hjkl13"),
+					String::from("test(x): test some stuff"),
+				),
 				Commit::new(
 					String::from("1234"),
 					String::from("fix: support preprocessing (fixes #99)"),
@@ -173,6 +188,9 @@ fn generate_changelog() -> Result<()> {
 - *(big-feature)* this is a breaking change
   - **BREAKING**: this is a breaking change
 
+### test
+- *(tests)* test some stuff
+
 ## Release v1.0.0
 
 ### chore
